searchData={"items":[{"type":"protocol","title":"Proximal","doc":"","ref":"Proximal.html"},{"type":"type","title":"Proximal.t/0","doc":"All the types that implement this protocol.","ref":"Proximal.html#t:t/0"},{"type":"function","title":"Proximal.to_xmlel/1","doc":"Transforms a data structure to Xmlel structure.","ref":"Proximal.html#to_xmlel/1"},{"type":"behaviour","title":"Proximal.Document","doc":"When you need to implement a parser for a structure, you could add the\nfollowing code:\n\n```elixir\ndefmodule MyStruct do\n  use Proximal.Document\n  alias Proximal.Xmlel\n\n  defstruct [:name, :surname]\n\n  @impl Proximal.Document\n  def render(%__MODULE__{name: name, surname: surname}) do\n    Xmlel.new(\"mystruct\", %{}, [\n      Xmlel.new(\"name\", %{}, [name]),\n      Xmlel.new(\"surname\", %{}, [surname])\n    ])\n  end\nend\n```\n\nThe code should let you to create your data in the way you need and\nthanks to the implementation of the `render/1` function, you could use:\n\n```elixir\n%MyStruct{name: \"Manuel\", surname: \"Rubio\"}\n|> Proximal.to_xmlel()\n```\n\nTo get the representation in `%Xmlel{}` structs and even adding at the\nend of that pipe `to_string/1` to get the XML in a string.","ref":"Proximal.Document.html"},{"type":"callback","title":"Proximal.Document.render/1","doc":"","ref":"Proximal.Document.html#c:render/1"},{"type":"module","title":"Proximal.Rpc","doc":"XML-RPC let us to create method calls and responses. It works in both\ndirections, encoding/decoding requests and encoding/decoding responses.","ref":"Proximal.Rpc.html"},{"type":"function","title":"Proximal.Rpc.decode_params/1","doc":"Decode params passed to the RPC system. It's performing the opposite as\n`encode_params/1` does.","ref":"Proximal.Rpc.html#decode_params/1"},{"type":"function","title":"Proximal.Rpc.decode_request/1","doc":"Decode a response from RPC. This performs the complementary of\n`encode_response/1`.\n\nExamples:\n    iex> import Proximal.Xmlel, only: [sigil_x: 2]\n    iex> ~x[  io_puts    Manuel     io_puts ]\n    iex> |> Proximal.Rpc.decode_request()\n    {\"io_puts\", [\"Manuel\"]}\n\n    iex> import Proximal.Xmlel, only: [sigil_x: 2]\n    iex> ~x[  io_puts     100      io_puts ]\n    iex> |> Proximal.Rpc.decode_request()\n    {\"io_puts\", [100]}\n\n    iex> import Proximal.Xmlel, only: [sigil_x: 2]\n    iex> ~x[  io_puts     AAAA      io_puts ]\n    iex> |> Proximal.Rpc.decode_request()\n    {\"io_puts\", [<<0, 0, 0>>]}\n\n    iex> import Proximal.Xmlel, only: [sigil_x: 2]\n    iex> ~x[  io_puts          io_puts ]\n    iex> |> Proximal.Rpc.decode_request()\n    {\"io_puts\", [nil]}\n\n    iex> import Proximal.Xmlel, only: [sigil_x: 2]\n    iex> ~x[  io_puts     10.5      io_puts ]\n    iex> |> Proximal.Rpc.decode_request()\n    {\"io_puts\", [10.5]}\n\n    iex> import Proximal.Xmlel, only: [sigil_x: 2]\n    iex> ~x[  io_puts     2019-03-31T02:30:00Z      io_puts ]\n    iex> |> Proximal.Rpc.decode_request()\n    {\"io_puts\", [~N[2019-03-31 02:30:00]]}\n\n    iex> import Proximal.Xmlel, only: [sigil_x: 2]\n    iex> ~x[  io_puts     2019-03-31T02:30:00Z      io_puts ]\n    iex> |> Proximal.Rpc.decode_request()\n    {\"io_puts\", [~N[2019-03-31 02:30:00]]}\n\n    iex> import Proximal.Xmlel, only: [sigil_x: 2]\n    iex> ~x[  io_puts       name   Manuel         io_puts ]\n    iex> |> Proximal.Rpc.decode_request()\n    {\"io_puts\", [%{\"name\" => \"Manuel\"}]}\n\n    iex> import Proximal.Xmlel, only: [sigil_x: 2]\n    iex> ~x[  io_puts        1    0    10         io_puts ]\n    iex> |> Proximal.Rpc.decode_request()\n    {\"io_puts\", [[true, false, 10]]}","ref":"Proximal.Rpc.html#decode_request/1"},{"type":"function","title":"Proximal.Rpc.decode_value/1","doc":"","ref":"Proximal.Rpc.html#decode_value/1"},{"type":"function","title":"Proximal.Rpc.encode_params/1","doc":"","ref":"Proximal.Rpc.html#encode_params/1"},{"type":"function","title":"Proximal.Rpc.encode_response/1","doc":"Encode a response. Providing a data, it's generating the XML-RPC valid\nresponse.\n\nExample:\n\n    iex> Proximal.Rpc.encode_response(100)\n    iex> |> to_string()\n    \"     100     \"\n\n    iex> Proximal.Rpc.encode_response(<<0, 0, 0>>)\n    iex> |> to_string()\n    \"     AAAA     \"\n\n    iex> Proximal.Rpc.encode_response(nil)\n    iex> |> to_string()\n    \"         \"\n\n    iex> Proximal.Rpc.encode_response(10.5)\n    iex> |> to_string()\n    \"     10.5     \"\n\n    iex> Proximal.Rpc.encode_response(DateTime.new!(~D[2019-03-31], ~T[02:30:00]))\n    iex> |> to_string()\n    \"     2019-03-31T02:30:00Z     \"\n\n    iex> Proximal.Rpc.encode_response(NaiveDateTime.new!(~D[2019-03-31], ~T[02:30:00]))\n    iex> |> to_string()\n    \"     2019-03-31T02:30:00Z     \"\n\n    iex> Proximal.Rpc.encode_response(%{\"name\" => \"Manuel\"})\n    iex> |> to_string()\n    \"       name   Manuel        \"\n\n    iex> Proximal.Rpc.encode_response([true, false, 10])\n    iex> |> to_string()\n    \"        1    0    10        \"","ref":"Proximal.Rpc.html#encode_response/1"},{"type":"function","title":"Proximal.Rpc.encode_value/1","doc":"","ref":"Proximal.Rpc.html#encode_value/1"},{"type":"module","title":"Proximal.Stream","doc":"Process a stream chunk by chunk to obtain a XML document.\nThe stream is initiated passing a PID where the information\nfrom the chunks are going to be received.\n\nThe parser is using `Proximal.Handler.Sender` which is\nresponsible to convert the stream data into a valid `%Xmlel{}`\nand send it back to the specified process.","ref":"Proximal.Stream.html"},{"type":"function","title":"Proximal.Stream.new/1","doc":"Creates a new `Saxy.Partial` struct to parser little by little\nthe incoming stream. The information is sent back to the `pid`\npassed as a paramter, by default this is set as `self()`.","ref":"Proximal.Stream.html#new/1"},{"type":"function","title":"Proximal.Stream.parse/2","doc":"Use it to send every chunk of the XML document(s) we want to\nparse. Every `chunk` will be sent back to the process passed\ninitially in the new function through the `partial` parameter.","ref":"Proximal.Stream.html#parse/2"},{"type":"function","title":"Proximal.Stream.terminate/1","doc":"When we wants to send the rest of the XML document(s) we\nhave to use this function which let us to finalise the\n`partial` data.\n\nNote that because a change into the Saxy 1.4, everytime we\nterminate a ending document, it will be in the `:halt` state,\notherwise it will generate a parsing error.","ref":"Proximal.Stream.html#terminate/1"},{"type":"module","title":"Proximal.Xmlel","doc":"Xmlel is a struct data which is intended to help with the parsing\nof the XML elements.","ref":"Proximal.Xmlel.html"},{"type":"type","title":"Proximal.Xmlel.attr_name/0","doc":"","ref":"Proximal.Xmlel.html#t:attr_name/0"},{"type":"type","title":"Proximal.Xmlel.attr_value/0","doc":"","ref":"Proximal.Xmlel.html#t:attr_value/0"},{"type":"type","title":"Proximal.Xmlel.attrs/0","doc":"","ref":"Proximal.Xmlel.html#t:attrs/0"},{"type":"type","title":"Proximal.Xmlel.children/0","doc":"","ref":"Proximal.Xmlel.html#t:children/0"},{"type":"function","title":"Proximal.Xmlel.children_tag_names/1","doc":"Provide the name of the children tags.\n\nExamples:\n    iex> import Proximal.Xmlel, only: [sigil_x: 2]\n    iex> xmlel = ~x[     ]\n    iex> Proximal.Xmlel.children_tag_names(xmlel)\n    [\"child1\", \"child2\", \"child3\"]","ref":"Proximal.Xmlel.html#children_tag_names/1"},{"type":"function","title":"Proximal.Xmlel.clean_spaces/1","doc":"This function removes the extra spaces inside of the stanzas starting from\n`xmlel` to ensure we can perform matching in a proper way.\n\nExamples:\n    iex> \" \\n     \\n        Hello world!\\n     \\n \"\n    iex> |> Proximal.Xmlel.parse()\n    iex> |> Proximal.Xmlel.clean_spaces()\n    iex> |> to_string()\n    \"  Hello world!  \"","ref":"Proximal.Xmlel.html#clean_spaces/1"},{"type":"function","title":"Proximal.Xmlel.decode/1","doc":"This function is a helper function to translate the tuples coming\nfrom Saxy into de `data` parameter to the `Proximal.Xmlel` structs.\n\nExamples:\n    iex> Proximal.Xmlel.decode({\"foo\", [], []})\n    %Proximal.Xmlel{full_name: \"foo\", name: \"foo\", attrs: %{}, children: []}\n\n    iex> Proximal.Xmlel.decode({\"foo\", [], [{:characters, \"1&1\"}]})\n    %Proximal.Xmlel{full_name: \"foo\", name: \"foo\", children: [\"1&1\"]}\n\n    iex> Proximal.Xmlel.decode({\"bar\", [{\"id\", \"10\"}], [\"Hello!\"]})\n    %Proximal.Xmlel{full_name: \"bar\", name: \"bar\", attrs: %{\"id\" => \"10\"}, children: [\"Hello!\"]}","ref":"Proximal.Xmlel.html#decode/1"},{"type":"function","title":"Proximal.Xmlel.delete_attr/2","doc":"Deletes an attribute by `name` from a `xmlel` struct.\n\nExamples:\n    iex> attrs = %{\"id\" => \"100\", \"name\" => \"Alice\"}\n    iex> xmlel = %Proximal.Xmlel{attrs: attrs}\n    iex> Proximal.Xmlel.get_attr(xmlel, \"name\")\n    \"Alice\"\n    iex> Proximal.Xmlel.delete_attr(xmlel, \"name\")\n    iex> |> Proximal.Xmlel.get_attr(\"name\")\n    nil","ref":"Proximal.Xmlel.html#delete_attr/2"},{"type":"function","title":"Proximal.Xmlel.encode/1","doc":"This function is a helper function to translate the content of the\n`xmlel` structs to the tuples needed by Saxy.\n\nExamples:\n    iex> Proximal.Xmlel.encode(%Proximal.Xmlel{name: \"foo\"})\n    {\"foo\", [], []}\n\n    iex> Proximal.Xmlel.encode(%Proximal.Xmlel{name: \"bar\", attrs: %{\"id\" => \"10\"}, children: [\"Hello!\"]})\n    {\"bar\", [{\"id\", \"10\"}], [{:characters, \"Hello!\"}]}\n\n    iex> Proximal.Xmlel.encode(%TestBuild{name: \"bro\"})\n    {\"bro\", [], []}","ref":"Proximal.Xmlel.html#encode/1"},{"type":"function","title":"Proximal.Xmlel.fetch/2","doc":"Access the value stored under `key` passing the stanza in\n`Proximal.Xmlel` format into the `xmlel` parameter.\n\nExamples:\n    iex> import Proximal.Xmlel\n    iex> el = ~x(     )\n    iex> fetch(el, \"c1\")\n    {:ok, [%Proximal.Xmlel{attrs: %{\"v\" => \"1\"}, children: [], name: \"c1\", full_name: \"c1\"}, %Proximal.Xmlel{attrs: %{\"v\" => \"2\"}, children: [], name: \"c1\", full_name: \"c1\"}]}\n    iex> fetch(el, \"nonexistent\")\n    :error","ref":"Proximal.Xmlel.html#fetch/2"},{"type":"function","title":"Proximal.Xmlel.get_and_update/3","doc":"Access the value under `key` and update it at the same time for the `xmlel`\nusing the `function` passed as paramter.\n\nExamples:\n    iex> import Proximal.Xmlel\n    iex> el = ~x(     )\n    iex> fun = fn els ->\n    iex> values = Enum.map(els, fn %Proximal.Xmlel{attrs: %{\"v\" => v}} = el -> %Proximal.Xmlel{el | attrs: %{\"v\" => \"v\" <> v}} end)\n    iex> {els, values}\n    iex> end\n    iex> get_and_update(el, \"c1\", fun)\n    {[%Proximal.Xmlel{attrs: %{\"v\" => \"1\"}, children: [], name: \"c1\", full_name: \"c1\"}, %Proximal.Xmlel{attrs: %{\"v\" => \"2\"}, children: [], name: \"c1\", full_name: \"c1\"}], %Proximal.Xmlel{attrs: %{}, children: [%Proximal.Xmlel{attrs: %{\"v\" => \"v1\"}, children: [], name: \"c1\", full_name: \"c1\"}, %Proximal.Xmlel{attrs: %{\"v\" => \"v2\"}, children: [], name: \"c1\", full_name: \"c1\"}, %Proximal.Xmlel{attrs: %{}, children: [], name: \"c2\", full_name: \"c2\"}], name: \"foo\", full_name: \"foo\"}}\n    iex> fun = fn _els -> :pop end\n    iex> get_and_update(el, \"c1\", fun)\n    {[%Proximal.Xmlel{attrs: %{\"v\" => \"1\"}, children: [], name: \"c1\", full_name: \"c1\"}, %Proximal.Xmlel{attrs: %{\"v\" => \"2\"}, children: [], name: \"c1\", full_name: \"c1\"}], %Proximal.Xmlel{attrs: %{}, children: [%Proximal.Xmlel{attrs: %{}, children: [], name: \"c2\", full_name: \"c2\"}], name: \"foo\", full_name: \"foo\"}}","ref":"Proximal.Xmlel.html#get_and_update/3"},{"type":"function","title":"Proximal.Xmlel.get_attr/3","doc":"Retrieve an attribute by `name` from a `xmlel` struct. If the value\nis not found the `default` value is used instead. If `default` is\nnot provided then `nil` is used as default value.\n\nExamples:\n    iex> attrs = %{\"id\" => \"100\", {\"ns1\", \"name\"} => \"Alice\"}\n    iex> xmlel = %Proximal.Xmlel{attrs: attrs}\n    iex> Proximal.Xmlel.get_attr(xmlel, \"name\")\n    \"Alice\"\n    iex> Proximal.Xmlel.get_attr(xmlel, \"id\")\n    \"100\"\n    iex> Proximal.Xmlel.get_attr(xmlel, \"ns1:name\")\n    \"Alice\"\n    iex> Proximal.Xmlel.get_attr(xmlel, \"surname\")\n    nil","ref":"Proximal.Xmlel.html#get_attr/3"},{"type":"function","title":"Proximal.Xmlel.new/3","doc":"Creates a Xmlel struct passing the `name` of the stanza, the `attrs`\nas a map or keyword list to create the attributes and `children` for\nthe payload of the XML tag. This is not recursive so it's intended\nthe children has to be in a correct format.\n\nThe children could be or binaries (strings) representing CDATA or other\n`Proximal.Xmlel` elements.\n\nExamples:\n    iex> Proximal.Xmlel.new(\"foo\")\n    %Proximal.Xmlel{full_name: \"foo\", attrs: %{}, children: [], name: \"foo\"}\n\n    iex> Proximal.Xmlel.new(\"bar\", %{\"id\" => \"10\"})\n    %Proximal.Xmlel{full_name: \"bar\", attrs: %{\"id\" => \"10\"}, children: [], name: \"bar\"}\n\n    iex> Proximal.Xmlel.new(\"bar\", [{\"id\", \"10\"}])\n    %Proximal.Xmlel{full_name: \"bar\", attrs: %{\"id\" => \"10\"}, children: [], name: \"bar\"}","ref":"Proximal.Xmlel.html#new/3"},{"type":"function","title":"Proximal.Xmlel.parse/1","doc":"Parser a `xml` string into `Proximal.Xmlel` struct.\n\nExamples:\n    iex> Proximal.Xmlel.parse(\" \")\n    {%Proximal.Xmlel{full_name: \"foo\", name: \"foo\", attrs: %{}, children: []}, \"\"}\n\n    iex> Proximal.Xmlel.parse(\" hello world! \")\n    {%Proximal.Xmlel{full_name: \"foo\", name: \"foo\", attrs: %{\"bar\" => \"10\"}, children: [\"hello world!\"]}, \"\"}\n\n    iex> Proximal.Xmlel.parse(\"  hello world!  \")\n    {%Proximal.Xmlel{full_name: \"foo\", name: \"foo\", attrs: %{}, children: [%Proximal.Xmlel{full_name: \"bar\", name: \"bar\", attrs: %{}, children: [\"hello world!\"]}]}, \"\"}\n\n    iex> Proximal.Xmlel.parse(\"  \")\n    {%Proximal.Xmlel{full_name: \"foo\", name: \"foo\", attrs: %{}, children: []}, \" \"}","ref":"Proximal.Xmlel.html#parse/1"},{"type":"function","title":"Proximal.Xmlel.pop/2","doc":"Pop the value under `key` passed an `Proximal.Xmlel` struct as `element`.\n\nExamples:\n    iex> import Proximal.Xmlel\n    iex> el = ~x(     )\n    iex> pop(el, \"c1\")\n    {[%Proximal.Xmlel{attrs: %{\"v\" => \"1\"}, children: [], name: \"c1\", full_name: \"c1\"}, %Proximal.Xmlel{attrs: %{\"v\" => \"2\"}, children: [], name: \"c1\", full_name: \"c1\"}], %Proximal.Xmlel{attrs: %{}, children: [%Proximal.Xmlel{attrs: %{}, children: [], name: \"c2\", full_name: \"c2\"}], name: \"foo\", full_name: \"foo\"}}\n    iex> pop(el, \"nonexistent\")\n    {[], %Proximal.Xmlel{attrs: %{}, children: [%Proximal.Xmlel{attrs: %{\"v\" => \"1\"}, children: [], name: \"c1\", full_name: \"c1\"}, %Proximal.Xmlel{attrs: %{\"v\" => \"2\"}, children: [], name: \"c1\", full_name: \"c1\"}, %Proximal.Xmlel{attrs: %{}, children: [], name: \"c2\", full_name: \"c2\"}], name: \"foo\", full_name: \"foo\"}}","ref":"Proximal.Xmlel.html#pop/2"},{"type":"function","title":"Proximal.Xmlel.put_attr/3","doc":"Add or set a `value` by `name` as attribute inside of the `xmlel` struct\npassed as parameter.\n\nExamples:\n    iex> attrs = %{\"id\" => \"100\", \"name\" => \"Alice\"}\n    iex> %Proximal.Xmlel{attrs: attrs}\n    iex> |> Proximal.Xmlel.put_attr(\"name\", \"Bob\")\n    iex> |> Proximal.Xmlel.get_attr(\"name\")\n    \"Bob\"","ref":"Proximal.Xmlel.html#put_attr/3"},{"type":"function","title":"Proximal.Xmlel.put_attrs/2","doc":"Add or set one or several attributes using `fields` inside of the `xmlel`\nstruct passed as parameter. The `fields` data are in keyword list format.\n\nExamples:\n    iex> fields = %{\"id\" => \"100\", \"name\" => \"Alice\", \"city\" => \"Cordoba\"}\n    iex> Proximal.Xmlel.put_attrs(%Proximal.Xmlel{name: \"foo\"}, fields) |> to_string()\n    \" \"\n\n    iex> fields = %{\"id\" => \"100\", \"name\" => \"Alice\", \"city\" => :\"Cordoba\"}\n    iex> Proximal.Xmlel.put_attrs(%Proximal.Xmlel{name: \"foo\"}, fields) |> to_string()\n    \" \"","ref":"Proximal.Xmlel.html#put_attrs/2"},{"type":"function","title":"Proximal.Xmlel.sigil_X/2","doc":"Sigil to use ~X to provide XML `string` and transform it to Xmlel struct.\nNote that we are not using `addons`.\n\nExamples:\n    iex> import Proximal.Xmlel\n    iex> ~X| \n    iex>  \n    iex> |\n    %Proximal.Xmlel{attrs: %{}, children: [\"\\n \"], name: \"foo\", full_name: \"foo\"}","ref":"Proximal.Xmlel.html#sigil_X/2"},{"type":"function","title":"Proximal.Xmlel.sigil_x/2","doc":"Sigil to use ~x to provide XML `string` and transform it to Xmlel struct\nremoving spaces and breaking lines.\nNote that we are not using `addons`.\n\nExamples:\n    iex> import Proximal.Xmlel\n    iex> ~x| \n    iex>  \n    iex> |\n    %Proximal.Xmlel{attrs: %{}, children: [], name: \"foo\", full_name: \"foo\"}","ref":"Proximal.Xmlel.html#sigil_x/2"},{"type":"type","title":"Proximal.Xmlel.t/0","doc":"Xmlel.`t` defines the `xmlel` element which contains the `name`, the\n`attrs` (attributes) and `children` for the XML tags.","ref":"Proximal.Xmlel.html#t:t/0"},{"type":"extras","title":"proXiMaL","doc":"# proXiMaL\n\n[![Build Status](https://github.com/altenwald/proximal/actions/workflows/elixir.yml/badge.svg)](https://github.com/altenwald/proximal/actions/workflows/elixir.yml)\n[![Coverage Status](https://coveralls.io/repos/github/altenwald/proximal/badge.svg)](https://coveralls.io/github/altenwald/proximal)\n[![License: LGPL 2.1](https://img.shields.io/github/license/altenwald/proximal.svg)](https://github.com/altenwald/proximal/blob/main/COPYING)\n[![Hex](https://img.shields.io/hexpm/v/proximal.svg)](https://hex.pm/packages/proximal)\n\n[Saxy]: https://github.com/qcam/saxy\n[Nerves]: https://www.nerves-project.org/\n\nproXiMaL is an advanced library for XML which let you to handle XML documents\nis a better way thanks to the Elixir idioms implemented.\n\nIt's built on top of [Saxy] which is providing the needed mechanisms to perform\nthe parsing and building of the XML documents.\n\nThis library is 100% Elixir. No C, Rust or other language implementations are\nneeded so, it's eligible to be in use for [Nerves] and embedded systems.","ref":"readme.html"},{"type":"extras","title":"Installation - proXiMaL","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `proximal` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:proximal, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}